require 'json'
require 'uu/os/uesuri_builder'

module UU
  module OS
    module GVC

      # Component for generating URIs for opening dialog windows.
      class Dialog

        # The use case to show in the dialog.
        attr_accessor :use_case_uri

        # The width of the dialog.
        attr_accessor :width

        # The height of the dialog.
        attr_accessor :height

        # The VUC action to trigger after the dialog is closed.
        attr_accessor :callback_action

        # Creates a new instance of Dialog which can be used for opening a dialog.
        # The Dialog is then typically converted to a UES-URI by using {UU::OS::GVC::Dialog#to_uri} method.
        #
        # @example
        #   # modify button so that it opens uc_uri in a dialog window with size
        #   # 400x300px and executes 'dialogClosed' event after the dialog closes
        #   button.action_uri = UU::OS::GVC::Dialog.new(uc_uri, {
        #     :width => 400,
        #     :height => 300,
        #     :callbackAction => 'dialogClosed'
        #   }).to_uri
        #
        # @example
        #   # init dialog
        #   dialog = UU::OS::GVC::Dialog.new('ues:TER:ART:?VENDOR.APP/PKG/UC&param=value')
        #   dialog.width = 400
        #   dialog.height = 300
        #   dialog.callback_action = 'dialogClosed'
        #   view.navigation.action = dialog.to_uri
        #
        # @param [String, UU::OS::UESURI] use_case_uri The UES-URI to be opened in the dialog window.
        # @param [Hash] window_parameters The dialog window parameters:
        #   * width - the width of the dialog
        #   * height - the height of the dialog. If not given, the dialog will size itself automatically.
        #   * callbackAction - the VUC action code to trigger in main window (from which the dialog was opened)
        #                      after the dialog gets closed
        def initialize(use_case_uri, window_parameters = nil)
          @use_case_uri = use_case_uri
          if !window_parameters.nil? && window_parameters.kind_of?(Hash)
            window_parameters.entries.each do |attribute|
              field_setter = get_setter(attribute[0])
              begin
                self.send(field_setter, attribute[1])
              rescue Exception => e
                raise e
              end
            end
          end
        end

        # Returns dialog as a String.
        #
        # @return [String] Dialog as String.
        def to_s
          to_uri
        end

        # Returns dialog as a URI.
        #
        #   dialog = UU::OS::GVC::Dialog.new(uc_uri, { :width => 400 })
        #   button.action_uri = dialog.to_uri
        #
        # @return [String] Data as String.
        def to_uri
          win_params = {
            :target => "DIALOG_WINDOW"
          }
          if !@width.nil? then win_params[:width] = @width; end
          if !@height.nil? then win_params[:height] = @height; end
          if !@callback_action.nil? then win_params[:callbackAction] = @callback_action; end

          uri_builder = UU::OS::UESURIBuilder.parse_uesuri(@use_case_uri)
          uri_builder = uri_builder.add_parameter('windowParameters', win_params.to_json)
          uri_builder.to_uesuri
        end

        private

        def get_setter(txt)
          result = txt.to_s
          result.gsub!(/([A-Z]+)([A-Z][a-z])/,'\1_\2')
          result.gsub!(/([a-z\d])([A-Z])/,'\1_\2')
          result.tr!('-', '_')
          (result.downcase << '=').to_sym
        end

      end
    end
  end
end
