require 'json'

module UU
  module OS
    module GVC

      # Representation of menu data as used by components with menu, such as drop-down button.
      class MenuModel

        # Creates a new instance of MenuModel.
        #
        # @example
        #   # init empty MenuModel, with no menu items
        #   mm = UU::OS::GVC::MenuModel.new()
        #
        # @example
        #   # init MenuModel as copy of another model
        #   menu_model = ... # get existing MenuModel
        #   mm_copy = UU::OS::GVC::MenuModel.new(menu_model)
        #
        # @example
        #   # init MenuModel from Array
        #
        #   array_data = [
        #                { :text => "Menu item 1", :actionUri => "ues:TER:ART:" },
        #                { :text => "Menu item 2", :actionUri => "ues:${this-art}?:myevent" },
        #                { :text => "Menu item 3", :disabled => true },
        #                {},
        #                { :text => "Menu item 4", :actionUri => "http://google.com/", :target => "NEW_WINDOW" }
        #              ]
        #   menu_model = UU::OS::GVC::MenuModel.new(array_data)
        #
        # @example
        #   # init MenuModel from String containing data in JSON Format
        #
        #   str_data = '[
        #                 {"text":"Menu item 1", "actionUri": "ues:TER:ART:"},
        #                 {"text":"Menu item 2", "actionUri": "ues:${this-art}?:myevent"},
        #                 {"text":"Menu item 3", "disabled": true},
        #                 {},
        #                 {"text":"Menu item 4", "actionUri": "http://google.com/", "target": "NEW_WINDOW"}
        #               ]'
        #   menu_model = UU::OS::GVC::MenuModel.new(str_data)
        #
        # @param [String, Array, UU::OS::GVC::MenuModel] data Initialization data in form of JSON String, Array or MenuModel.
        #   * String - String with valid JSON containing data in Array Format.
        #   * Array - Array containing data in Array Format.
        #   * {UU::OS::GVC::MenuModel} - Another MenuModel, new model will be copy of passed data.
        # @raise [ArgumentError] If data is not one of JSON String, Array or MenuModel.
        def initialize(data=nil)
          if data.nil?
            @json = nil
          elsif data.kind_of? String
            js = JSON.parse(data, :symbolize_names => true)
            if (!js.kind_of? Array)
              raise ArgumentError.new("JSON String has bad format. The root element must be Array but was #{js.class}.")
            end
            init_array(js)
          elsif data.kind_of? Array
            init_array(data)
          elsif data.kind_of? MenuModel
            @json = data.to_a
          else
            raise ArgumentError.new("MenuModel data must be JSON String, Array or MenuModel, but was #{data.class}.")
          end
        end

        # Adds a new item to the end of Menu, and returns the index of the new item.
        #
        # @example
        #   # add item using JSON String
        #   item = '{"text": "Menu item 1", "actionUri": "ues:TER:ART:"}'
        #   mm.add_item(item)
        #
        #   # add same item using Hash
        #   item = { :text => "Menu item 1", :actionUri => "ues:TER:ART:" }
        #   mm.add_item(item)
        #
        # @param [String, Hash, MenuItem] menu_item Menu item to add.
        # @return [Numeric] Index of the new item.
        # @raise [ArgumentError] If row has bad format or is not one of String, Hash or MenuItem.
        def add_item(menu_item)
          item = MenuItem.new(menu_item)
          @json = [] if @json.nil?
          @json << item
          @json.size - 1
        end

        # Returns menu item at specified index. Returned object is of MenuItem class. Modification of the returned
        # object has no influence on the original data of Menu.
        #
        # @example
        #   # get third row
        #   item = mm.get_item(2)
        #
        #   # get last row
        #   item = mm.get_item(-1)
        #
        # @param [Integer] index Index of menu item. An index can be represented by a single positive integer between 0 (index 0 = first row)
        #   and the number of rows (excluding) which is returned by {UU::OS::GVC::MenuModel#number_of_items} method.
        #   You can use negative indices as well, which start counting from the end (i.e. minus number of rows), with -1 being the last element.
        # @return [MenuItem] Menu item at specified index.
        # @raise [ArgumentError] If index is greater than or equal to number of rows.
        def get_item(index)
          check_index(index, -number_of_items, number_of_items-1, "index")
          deep_copy(@json[index])
        end

        # Returns number of items in the menu. May be zero.
        #
        # @example
        #   # get number of items in the menu
        #   items_count = mm.number_of_items
        #
        # @return [Numeric] Number of items.
        def number_of_items
          if @json.nil?
            0
          else
            @json.size
          end
        end

        # Inserts a new item to the menu at specified index. All existing items from specified index to the end
        # are shifted (index is increased by one).
        #
        # @example
        #   # insert item at fifth position using JSON String
        #   item = '{"text": "Menu item 1", "actionUri": "ues:TER:ART:"}'
        #   mm.insert_item(4, item)
        #
        #   # insert same item using Hash
        #   item = { :text => "Menu item 1", :actionUri => "ues:TER:ART:" }
        #   mm.insert_item(4, item)
        #
        # @param [Integer] index Index where new item will be inserted. An index can be represented by a single positive integer between 0 (index 0 = first row)
        #   and the number of items (excluding) which is returned by {UU::OS::GVC::MenuModel#number_of_items} method.
        #   You can use negative indices as well, which start counting from the end (i.e. minus number of items), with -1 being the last element.
        #   Inserting with negative index adds the new item after the element.
        # @param [String, Hash, MenuItem] menu_item Menu item.
        # @return [void]
        # @raise [ArgumentError] If item has bad format or is not one of String, Hash or MenuItem. If index is greater than or equal to number of rows.
        def insert_item(index, menu_item)
          check_index(index, -number_of_items, number_of_items, "index")
          item = MenuItem.new(menu_item)
          @json = [] if @json.nil?
          @json.insert(index, item)
          return
        end

        # Set item at specific index, the original item at specified index will be overwritten.
        #
        # @example
        #   # set item at second position
        #   item = '{"text": "Menu item 1", "actionUri": "ues:TER:ART:"}'
        #   mm.set_item(1 ,item)
        #
        #   # set same item using Hash format
        #   item = { :text => "Menu item 1", :actionUri => "ues:TER:ART:" }
        #   mm.set_item(1 ,item)
        #
        # @param [Integer] index Index of item. An index can be represented by a single positive integer between 0 (index 0 = first row)
        #   and the number of rows (excluding) which is returned by {UU::OS::GVC::MenuModel#number_of_items} method.
        #   You can use negative indices as well, which start counting from the end (i.e. minus number of items), with -1 being the last element.
        # @param [String, Hash, MenuItem] menu_item Menu item.
        # @return [void]
        # @raise [ArgumentError] If item has bad format or is not one of String, Hash or MenuItem. If index is greater than or equal to number of items.
        def set_item(index, menu_item)
          check_index(index, -number_of_items, number_of_items-1, "index")
          item = MenuItem.new(menu_item)
          @json[index] = item
          return
        end

        # Removes the item at specific index. All existing items after specified index are shifted (index is decreased by one).
        #
        # @example
        #   # remove second item
        #   dtm.remove_item(1)
        #
        #   # remove last item
        #   dtm.remove_item(-1)
        #
        #
        # @param [Integer] index Index where menu item for removal is at. An index can be represented by a single positive integer between 0 (index 0 = first row)
        #   and the number of rows (excluding) which is returned by {UU::OS::GVC::MenuModel#number_of_items} method.
        #   You can use negative indices as well, which start counting from the end (i.e. minus number of items), with -1 being the last element.
        # @return [void]
        # @raise [ArgumentError] If index is greater than or equal to number of items.
        def remove_item(index)
          check_index(index, -number_of_items, number_of_items-1, "index")
          @json.delete_at(index)
          return
        end

        # Returns copy of data representation as MenuModel.
        #
        # @example
        #   # copy MenuModel
        #   mm_copy = mm.clone
        #
        #   mm.number_of_items # 5
        #   mm_copy.number_of_items # 5
        #
        #   # add item to original MenuModel
        #   mm.add_item({ :text => "Another item", :actionUri => "ues:TER:ART:" })
        #
        #   mm.number_of_items # 6
        #   mm_copy.number_of_items # 5
        #
        # @return [MenuModel] Copy of this MenuModel.
        def clone
          MenuModel.new(self)
        end

        # Returns copy of data representation as Array of MenuItem-s. Modification of the returned array
        # object has no influence on the original data of MenuModel.
        #
        # @example
        #   # get data as Array
        #   array = mm.to_a
        #
        # @return [Array] Data as Array of MenuItem-s.
        def to_a
          deep_copy(@json)
        end

        # Returns data representation as JSON String.
        #
        # @example
        #   # get JSON String
        #   data_str = mm.to_json
        #
        #   # output:
        #   # [
        #   #   {"text":"Menu item 1", "actionUri": "ues:TER:ART:"},
        #   #   {"text":"Menu item 2", "actionUri": "ues:${this-art}?:myevent"},
        #   #   {"text":"Menu item 3", "disabled": true},
        #   #   {},
        #   #   {"text":"Menu item 4", "actionUri": "http://google.com/", "target": "NEW_WINDOW"}
        #   # ]
        #
        # @return [String] Data as JSON String.
        def to_json(options = {})
          @json.to_json(options)
        end

        # Returns data representation as String.
        #
        # @example
        #   # get String
        #   data_str = mm.to_s
        #
        #   # output:
        #   # [
        #   #   {"text":"Menu item 1", "actionUri": "ues:TER:ART:"},
        #   #   {"text":"Menu item 2", "actionUri": "ues:${this-art}?:myevent"},
        #   #   {"text":"Menu item 3", "disabled": true},
        #   #   {},
        #   #   {"text":"Menu item 4", "actionUri": "http://google.com/", "target": "NEW_WINDOW"}
        #   # ]
        #
        # @return [String] Data as String.
        def to_s
          to_json
        end

        # Represenatation of a single menu item.
        class MenuItem

          # Item text.
          attr_accessor :text

          # Defines target window where the action URI will be opened - one of "CURRENT_WINDOW" (default),
          # "NEW_WINDOW", "DIALOG_WINDOW". For backward compatibility it's also possible to use older equivalents
          # "SELF", "NEW" and "MODAL".
          attr_accessor :target

          # URI to use for the menu item navigation (UESURI, URL). Can be also
          # UESURI with VUC event, such as "ues:${this-art}?:myevent".
          attr_accessor :action_uri

          # Whether the item is disabled.
          attr_accessor :disabled

          # Creates a new instance of MenuItem. Menu item with all fields set to nil
          # is considered a separator.
          #
          # @example
          #   # init empty menu item (separator)
          #   mi = UU::OS::GVC::MenuModel::MenuItem.new()
          #
          # @example
          #   # init MenuItem as copy of another item
          #   menu_item = ... # get existing MenuItem
          #   mi_copy = UU::OS::GVC::MenuModel::MenuItem.new(menu_item)
          #
          # @example
          #   # init MenuModel from Hash
          #   hash_data = { :text => "Menu item 1", :actionUri => "ues:TER:ART:" }
          #   mi = UU::OS::GVC::MenuModel::MenuItem.new(hash_data)
          #
          # @example
          #   # init MenuModel from String containing data in JSON Format
          #   str_data = '{"text":"Menu item 1", "actionUri": "ues:TER:ART:"}'
          #   mi = UU::OS::GVC::MenuModel::MenuItem.new(str_data)
          #
          # @param [String, Hash, UU::OS::GVC::MenuModel::MenuItem] data Initialization data.
          #   * String - String with valid JSON.
          #   * Hash - Hash containing data for menu item.
          #   * {UU::OS::GVC::MenuModel::MenuItem} - Another MenuItem, new item will be copy of passed data.
          # @raise [ArgumentError] If data is not one of JSON String, Hash or MenuItem.
          def initialize(data=nil)
            @attributes = {}
            if data.kind_of? String
              js = JSON.parse(data, :symbolize_names => true)
              if (!js.kind_of? Hash)
                raise ArgumentError.new("JSON String has bad format. The root element must be Hash but was #{js.class}.")
              end
              init_hash(js)
            elsif data.kind_of? Hash
              init_hash(deep_copy(data))
            elsif data.kind_of? MenuItem
              init_hash(data.send(:to_h))
            elsif !data.nil?
              raise ArgumentError.new("MenuItem data must be JSON String, Hash or MenuItem, but was #{data.class}.")
            end
          end

          # Returns data representation as JSON String.
          #
          # @example
          #   # get JSON String
          #   data_str = mi.to_json
          #
          #   # output:
          #   # {"text":"Menu item 1", "actionUri": "ues:TER:ART:"}
          #
          # @return [String] Data as JSON String.
          def to_json(options = {})
            @attributes.to_json(options)
          end

          # Returns data representation as String.
          #
          # @example
          #   # get String
          #   data_str = mi.to_s
          #
          #   # output:
          #   # {"text":"Menu item 1", "actionUri": "ues:TER:ART:"}
          #
          # @return [String] Data as String.
          def to_s
            to_json
          end

          def text
            @attributes[:text]
          end
          def text=(value)
            @attributes[:text] = value
          end
          def disabled
            @attributes[:disabled]
          end
          def disabled=(value)
            @attributes[:disabled] = value
          end
          def action_uri
            @attributes[:actionUri]
          end
          def action_uri=(value)
            @attributes[:actionUri] = value
          end
          def target
            @attributes[:target]
          end
          def target=(value)
            @attributes[:target] = value
          end


          private

          def to_h
            deep_copy(@attributes)
          end
          def init_hash(h)
            @attributes = h
            if @attributes.has_key?(:action_uri) # rename :action_uri to :actionUri in the hash
              @attributes[:actionUri] = @attributes[:action_uri] unless @attributes.has_key?(:actionUri)
              @attributes.delete(:action_uri)
            end
          end
          def deep_copy(object)
            Marshal.load(Marshal.dump(object))
          end
        end

        private

        def init_array(a)
          if (a.nil?) || (a.size == 0)
            @json = if a.nil? then nil else [] end
            return
          end
          @json = []
          a.each { |item| add_item(item) }
        end

        def check_index(index, min, max, parameter_name)
          if index < min || index > max
            raise ArgumentError.new("Parameter #{parameter_name} is out of bounds. Value must be between #{min} and #{max} (inclusive), but was #{index}.")
          end
        end

        def deep_copy(object)
          return Marshal.load(Marshal.dump(object))
        end
      end
    end
  end
end